@@ -134,6 +134,7 @@ class ModuleFinder(object):
         self.includeFiles = includeFiles
         if includeFiles is None:
             self.includeFiles = []
+        self.excludeDependentFiles = {}
         self.excludes = dict.fromkeys(excludes)
         self.replacePaths = replacePaths
         if replacePaths is None:
@@ -218,7 +219,7 @@ class ModuleFinder(object):
                 # Namespace package (?)
                 module = sys.modules[name]
                 info = ("", "", imp.PKG_DIRECTORY)
-                return None, module.__path__[0], info
+                return None, list(module.__path__)[0], info
 
             # Check for modules in zip files.
             # If a path is a subdirectory within a zip file, we must have
@@ -278,7 +279,7 @@ class ModuleFinder(object):
                 subModule = self._InternalImportModule(subModuleName,
                                 deferredImports)
                 if subModule is None:
-                    if subModule not in self.excludes:
+                    if subModuleName not in self.excludes:
                         raise ImportError("No module named %r" % subModuleName)
                 else:
                     module.globalNames[name] = None
@@ -539,38 +540,27 @@ class ModuleFinder(object):
         """Scan code, looking for imported modules and keeping track of the
            constants that have been created in order to better tell which
            modules are truly missing."""
-        opIndex = 0
         arguments = []
-        code = co.co_code
-        numOps = len(code)
-        is3 = sys.version_info[0] >= 3
-        while opIndex < numOps:
-            if is3:
-                op = code[opIndex]
-            else:
-                op = ord(code[opIndex])
-            opIndex += 1
-            if op >= dis.HAVE_ARGUMENT:
-                if is3:
-                    opArg = code[opIndex] + code[opIndex + 1] * 256
-                else:
-                    opArg = ord(code[opIndex]) + ord(code[opIndex + 1]) * 256
-                opIndex += 2
-            
+        importedModule = None
+        method = dis._unpack_opargs if sys.version_info[:3] >= (3, 5, 2) \
+                else self._UnpackOpArgs
+        for opIndex, op, opArg in method(co.co_code):
+
+            # keep track of constants (these are used for importing)
+            # immediately restart loop so arguments are retained
             if op == LOAD_CONST:
-                # Store an argument to be used later by an IMPORT_NAME operation.
                 arguments.append(co.co_consts[opArg])
-            
+                continue
+
+            # import statement: attempt to import module
             elif op == IMPORT_NAME:
                 name = co.co_names[opArg]
                 if len(arguments) >= 2:
                     relativeImportIndex, fromList = arguments[-2:]
                 else:
                     relativeImportIndex = -1
-                    fromList, = arguments
-                
+                    fromList = arguments[0] if arguments else []
                 if name not in module.excludeNames:
-                    # Load the imported module
                     importedModule = self._ImportModule(name, deferredImports,
                             module, relativeImportIndex)
                     if importedModule is not None:
@@ -578,51 +568,59 @@ class ModuleFinder(object):
                                 and importedModule.path is not None:
                             self._EnsureFromList(module, importedModule,
                                     fromList, deferredImports)
-            
-            elif op == IMPORT_FROM and topLevel:
-                if is3:
-                    op = code[opIndex]
-                    opArg = code[opIndex + 1] + code[opIndex + 2] * 256
-                else:
-                    op = ord(code[opIndex])
-                    opArg = ord(code[opIndex + 1]) + \
-                            ord(code[opIndex + 2]) * 256
-                opIndex += 3
-                if op == STORE_FAST:
-                    name = co.co_varnames[opArg]
-                else:
-                    name = co.co_names[opArg]
-                storeName = True
-                if deferredImports:
-                    deferredCaller, deferredPackage, deferredFromList = \
-                            deferredImports[-1]
-                    storeName = deferredCaller is not module
-                if storeName:
-                    module.globalNames[name] = None
-            
+
+            # import * statement: copy all global names
             elif op == IMPORT_STAR and topLevel and importedModule is not None:
                 module.globalNames.update(importedModule.globalNames)
-                arguments = []
-            
-            elif op not in (BUILD_LIST, INPLACE_ADD):
-                # The stack was used for something else, so we clear it.
-                if topLevel and op in STORE_OPS:
-                    name = co.co_names[opArg]
-                    module.globalNames[name] = None
-                arguments = []
-        
+
+            # store operation: track only top level
+            elif topLevel and op in STORE_OPS:
+                name = co.co_names[opArg]
+                module.globalNames[name] = None
+
+            # reset arguments; these are only needed for import statements so
+            # ignore them in all other cases!
+            arguments = []
+
         # Scan the code objects from function & class definitions
         for constant in co.co_consts:
             if isinstance(constant, type(co)):
                 self._ScanCode(constant, module, deferredImports,
                         topLevel = False)
 
+    def _UnpackOpArgs(self, code):
+        """Unpack the operations and arguments from the byte code. From Python
+           3.5 onwards this is found in the private method _unpack_opargs
+           but for earlier releases this wasn't available as a separate
+           method."""
+        opIndex = 0
+        numOps = len(code)
+        is3 = sys.version_info[0] >= 3
+        while opIndex < numOps:
+            offset = opIndex
+            if is3:
+                op = code[opIndex]
+            else:
+                op = ord(code[opIndex])
+            opIndex += 1
+            arg = None
+            if op >= dis.HAVE_ARGUMENT:
+                if is3:
+                    arg = code[opIndex] + code[opIndex + 1] * 256
+                else:
+                    arg = ord(code[opIndex]) + ord(code[opIndex + 1]) * 256
+                opIndex += 2
+            yield (offset, op, arg)
+
     def AddAlias(self, name, aliasFor):
         """Add an alias for a particular module; when an attempt is made to
            import a module using the alias name, import the actual name
            instead."""
         self.aliases[name] = aliasFor
 
+    def ExcludeDependentFiles(self, fileName):
+        self.excludeDependentFiles[fileName] = None
+
     def ExcludeModule(self, name):
         """Exclude the named module from the resulting frozen executable."""
         self.excludes[name] = None
@@ -640,9 +638,11 @@ class ModuleFinder(object):
         self._ImportDeferredImports(deferredImports)
         return module
 
-    def IncludeFiles(self, sourcePath, targetPath):
+    def IncludeFiles(self, sourcePath, targetPath, copyDependentFiles = True):
         """Include the files in the given directory in the target build."""
         self.includeFiles.append((sourcePath, targetPath))
+        if not copyDependentFiles:
+            self.ExcludeDependentFiles(sourcePath)
 
     def IncludeModule(self, name, namespace = False):
         """Include the named module in the frozen executable."""
